/* gInfo - Copyright (C) 2004 bakunin - Andrea Marchesini 
 *                                      <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include <editor.h>

enum
{
  TITLE_COLUMN,
  VALUE_COLUMN,
  NUM_COLUMNS
};

static GtkWidget *tagslist_tree;
static GtkTreeStore *tagslist_model;

void tools_tagslist_close (GtkWidget *, gpointer);
void tools_tagslist_row_activate (GtkTreeView *, GtkTreePath *,
				  GtkTreeViewColumn *);
void tools_tagslist_create_tree (void);

/* the wordcount window */
void
tools_wordcount (GtkWidget * w, gpointer dummy)
{
  char *txt;
  PangoLogAttr *attrs;
  int words = 0;
  int chars = 0;
  int white = 0;
  int lines = 0;
  int bytes = 0;
  register int i;

  char buf[1024];
  GtkWidget *dialog;
  GtkWidget *label;
  GtkWidget *table;
  GtkTextIter start, end;

  gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (sourcebuffer), &start,
				      0);
  gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (sourcebuffer), &end);

  txt =
    gtk_text_buffer_get_slice (GTK_TEXT_BUFFER (sourcebuffer), &start, &end,
			       TRUE);

  lines = gtk_text_buffer_get_line_count (GTK_TEXT_BUFFER (sourcebuffer));

  chars = g_utf8_strlen (txt, -1);
  attrs = g_new0 (PangoLogAttr, chars + 1);

  pango_get_log_attrs (txt, -1, 0, pango_language_from_string ("C"), attrs,
		       chars + 1);

  for (i = 0; i < chars; i++)
    {
      if (attrs[i].is_white)
	white++;
      if (attrs[i].is_word_start)
	words++;
    }

  if (chars == 0)
    lines = 0;

  bytes = strlen (txt);

  snprintf (buf, 1024, "%s %s - Word Count", NAME, VERSION);
  dialog =
    gtk_dialog_new_with_buttons (buf, GTK_WINDOW (editorwin),
				 GTK_DIALOG_MODAL |
				 GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK,
				 GTK_RESPONSE_OK, NULL);

  table = gtk_table_new (0, 0, FALSE);
  gtk_widget_show (table);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), table, FALSE,
		      FALSE, 0);

  snprintf (buf, 1024, "File name:");
  label = gtk_label_new (buf);
  gtk_widget_show (label);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);

  snprintf (buf, 1024, "<span weight=\"bold\">%s</span>",
	    editor_output ? editor_output : "Unknow");
  label = gtk_label_new (buf);
  gtk_widget_show (label);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, 0, 1);

  snprintf (buf, 1024, "Chars:");
  label = gtk_label_new (buf);
  gtk_widget_show (label);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);

  snprintf (buf, 1024, "<span weight=\"bold\">%d</span>", chars);
  label = gtk_label_new (buf);
  gtk_widget_show (label);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, 1, 2);

  snprintf (buf, 1024, "Lines:");
  label = gtk_label_new (buf);
  gtk_widget_show (label);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 2, 3);

  snprintf (buf, 1024, "<span weight=\"bold\">%d</span>", lines);
  label = gtk_label_new (buf);
  gtk_widget_show (label);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, 2, 3);

  snprintf (buf, 1024, "Words:");
  label = gtk_label_new (buf);
  gtk_widget_show (label);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 3, 4);

  snprintf (buf, 1024, "<span weight=\"bold\">%d</span>", words);
  label = gtk_label_new (buf);
  gtk_widget_show (label);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, 3, 4);

  snprintf (buf, 1024, "Chars non-spaces:");
  label = gtk_label_new (buf);
  gtk_widget_show (label);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 4, 5);

  snprintf (buf, 1024, "<span weight=\"bold\">%d</span>", chars - white);
  label = gtk_label_new (buf);
  gtk_widget_show (label);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, 4, 5);

  free (attrs);
  free (txt);

  gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);
}

/* The tagslist window */
void
tools_tagslist (GtkWidget * w, gpointer dummy)
{
  static GtkWidget *win = NULL;
  GtkWidget *box;
  GtkWidget *hbox;
  GtkWidget *label;
  GtkWidget *image;
  GtkWidget *button;
  char buf[1024];
  GtkWidget *sw;

  GtkTreeSelection *selection;
  GtkCellRenderer *cell;
  GtkTreeViewColumn *column;

  if (win)
    return;

  snprintf (buf, 1024, "%s %s - tags list", NAME, VERSION);
  win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (win), buf);
  gtk_widget_set_size_request (win, 300, 600);

  g_signal_connect ((gpointer) win, "destroy",
		    G_CALLBACK (tools_tagslist_close), &win);

  box = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (box);
  gtk_container_add (GTK_CONTAINER (win), box);

  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (sw);
  gtk_box_pack_start (GTK_BOX (box), sw, TRUE, TRUE, 0);

  tagslist_model =
    gtk_tree_store_new (NUM_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);

  tagslist_tree = gtk_tree_view_new ();
  gtk_widget_show (tagslist_tree);
  gtk_tree_view_set_model (GTK_TREE_VIEW (tagslist_tree),
			   GTK_TREE_MODEL (tagslist_model));

  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tagslist_tree));

  gtk_container_add (GTK_CONTAINER (sw), tagslist_tree);

  cell = gtk_cell_renderer_text_new ();

  column =
    gtk_tree_view_column_new_with_attributes ("Tags List", cell, "text",
					      TITLE_COLUMN, NULL);

  gtk_tree_view_append_column (GTK_TREE_VIEW (tagslist_tree),
			       GTK_TREE_VIEW_COLUMN (column));

  g_signal_connect (tagslist_tree, "row_activated",
		    G_CALLBACK (tools_tagslist_row_activate), tagslist_model);

  button = gtk_button_new ();
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (box), button, FALSE, TRUE, 0);

  g_signal_connect ((gpointer) button, "clicked",
		    G_CALLBACK (tools_tagslist_close), &win);

  hbox = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox);
  gtk_container_add (GTK_CONTAINER (button), hbox);

  image = gtk_image_new_from_stock ("gtk-close", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image);
  gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, TRUE, 0);

  label = gtk_label_new_with_mnemonic ("_Close");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);

  tools_tagslist_create_tree ();
  gtk_tree_view_expand_all (GTK_TREE_VIEW (tagslist_tree));

  gtk_widget_show (win);
}

/* Close the tagslist window */
void
tools_tagslist_close (GtkWidget * w, gpointer dummy)
{
  GtkWidget **win = (GtkWidget **) dummy;

  gtk_widget_destroy (*win);
  *win = NULL;
}

/* get the click in taglist tree */
void
tools_tagslist_row_activate (GtkTreeView * tree, GtkTreePath * path,
			     GtkTreeViewColumn * column)
{
  GtkTreeIter iter;
  GtkTreeModel *model;
  char *value;

  model = gtk_tree_view_get_model (tree);

  gtk_tree_model_get_iter (model, &iter, path);
  gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, VALUE_COLUMN, &value,
		      -1);

  if (value)
    gtk_text_buffer_insert_at_cursor (GTK_TEXT_BUFFER (sourcebuffer), value,
				      strlen (value));
}

/* Create the tagslist tree */
void
tools_tagslist_create_tree (void)
{

  GtkTreeIter iter;
  GtkTreeIter iter_c;
  GtkTreeIter iter_c1;
  GtkTreeIter iter_c2;
  GtkTreeIter iter_c3;
  GtkTreeIter iter_c4;
  GtkTreeIter iter_c5;

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter, NULL);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter, TITLE_COLUMN,
		      "Initialize", VALUE_COLUMN, NULL, -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c, &iter);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c, TITLE_COLUMN,
		      "XML init", VALUE_COLUMN,
		      "<?xml version=\"1.0\" standalone=\"no\"?>\n"
		      "<!DOCTYPE ginfo SYSTEM \"http://autistici.org/bakunin/ginfo-0.2.dtd\">",
		      -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c, &iter);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c, TITLE_COLUMN,
		      "gInfo root element", VALUE_COLUMN, "<ginfo></ginfo>",
		      -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c1, &iter_c);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c1, TITLE_COLUMN,
		      "module title", VALUE_COLUMN,
		      "<module default=\"0\">TITLE</module>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c1, &iter_c);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c1, TITLE_COLUMN,
		      "genre tag", VALUE_COLUMN, "<genre>GENRE</genre>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c1, &iter_c);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c1, TITLE_COLUMN,
		      "words tag", VALUE_COLUMN, "<words>WORDS</words>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c1, &iter_c);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c1, TITLE_COLUMN,
		      "tooltips section", VALUE_COLUMN,
		      "<tooltips></tooltips>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c2, &iter_c1);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c2, TITLE_COLUMN,
		      "tip tag", VALUE_COLUMN, "<tip>TIPS!</tip>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c1, &iter_c);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c1, TITLE_COLUMN,
		      "docs section", VALUE_COLUMN, "<docs></docs>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c2, &iter_c1);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c2, TITLE_COLUMN,
		      "doc subsection", VALUE_COLUMN, "<doc id=ID></doc>",
		      -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c3, &iter_c2);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c3, TITLE_COLUMN,
		      "chapter tag", VALUE_COLUMN,
		      "<chapter>CHAPTER</chapter>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c3, &iter_c2);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c3, TITLE_COLUMN,
		      "info tag", VALUE_COLUMN, "<info>INFO</info>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c4, &iter_c3);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c4, TITLE_COLUMN,
		      "Style tags", VALUE_COLUMN, NULL, -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "oblique tag", VALUE_COLUMN, "<oblique>TEXT</oblique>",
		      -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "italic tag", VALUE_COLUMN, "<italic>TEXT</italic>",
		      -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c4, &iter_c3);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c4, TITLE_COLUMN,
		      "Variant tags", VALUE_COLUMN, NULL, -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "small_caps tag", VALUE_COLUMN,
		      "<small_caps>TEXT</small_caps>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c4, &iter_c3);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c4, TITLE_COLUMN,
		      "Weight tags", VALUE_COLUMN, NULL, -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "ultralight tag", VALUE_COLUMN,
		      "<ultralight>TEXT</ultralight>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "light tag", VALUE_COLUMN, "<light>TEXT</light>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "bold tag", VALUE_COLUMN, "<bold>TEXT</bold>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "ultrabold tag", VALUE_COLUMN,
		      "<ultrabold>TEXT</ultrabold>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "heavy tag", VALUE_COLUMN, "<heavy>TEXT</heavy>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c4, &iter_c3);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c4, TITLE_COLUMN,
		      "Stretch tags", VALUE_COLUMN, NULL, -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "ultracondensed tag", VALUE_COLUMN,
		      "<ultracondensed>TEXT</ultracondensed>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "extracondensed tag", VALUE_COLUMN,
		      "<extracondensed>TEXT</extracondensed>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "condensed tag", VALUE_COLUMN,
		      "<condensed>TEXT</condensed>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "semicondensed tag", VALUE_COLUMN,
		      "<semicondensed>TEXT</semicondensed>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "semiexpanded tag", VALUE_COLUMN,
		      "<semiexpanded>TEXT</semiexpanded>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "expanded tag", VALUE_COLUMN,
		      "<expanded>TEXT</expanded>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "extraexpanded tag", VALUE_COLUMN,
		      "<extraexpanded>TEXT</extraexpanded>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "ultraexpanded tag", VALUE_COLUMN,
		      "<ultraexpanded>TEXT</ultraexpanded>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c4, &iter_c3);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c4, TITLE_COLUMN,
		      "Scale tags", VALUE_COLUMN, NULL, -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "xx_small tag", VALUE_COLUMN,
		      "<xx_small>TEXT</xx_small>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "x_small tag", VALUE_COLUMN, "<x_small>TEXT</x_small>",
		      -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "small tag", VALUE_COLUMN, "<small>TEXT</small>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "large tag", VALUE_COLUMN, "<large>TEXT</large>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "x_large tag", VALUE_COLUMN, "<x_large>TEXT</x_large>",
		      -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "xx_large tag", VALUE_COLUMN,
		      "<xx_large>TEXT</xx_large>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c4, &iter_c3);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c4, TITLE_COLUMN,
		      "size tag", VALUE_COLUMN,
		      "<size value=\"20\">TEXT</size>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c4, &iter_c3);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c4, TITLE_COLUMN,
		      "Margins tags", VALUE_COLUMN,
		      "<margin left=\"20\" right=\"20\">TEXT</margin>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c4, &iter_c3);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c4, TITLE_COLUMN,
		      "Color/Image tags", VALUE_COLUMN, NULL, -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "image tag", VALUE_COLUMN,
		      "<image>IMAGE_FILE</image>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "foreground tag", VALUE_COLUMN,
		      "<foreground value=\"COLOR\">TEXT</foreground>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "background tag", VALUE_COLUMN,
		      "<background value=\"COLOR\">TEXT</background>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c4, &iter_c3);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c4, TITLE_COLUMN,
		      "justify tag", VALUE_COLUMN,
		      "<justify value=\"LEFT, RIGHT or CENTER\">TEXT</justify>",
		      -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c4, &iter_c3);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c4, TITLE_COLUMN,
		      "Underline tags", VALUE_COLUMN, NULL, -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "underline tag", VALUE_COLUMN,
		      "<underline>TEXT</underline>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "double_underline tag", VALUE_COLUMN,
		      "<double_underline>TEXT</double_underline>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c4, &iter_c3);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c4, TITLE_COLUMN,
		      "face tag", VALUE_COLUMN,
		      "<face value=\"FONT\">TEXT</face>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "font tag", VALUE_COLUMN, "<font>TEXT</font>", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "variant font element", VALUE_COLUMN,
		      "variant=\"small_caps\" ", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "weight font element", VALUE_COLUMN,
		      "weight=\"ultracondensed, extracondensed, condensed, semicondensed, semiexpanded, expanded, estraexpanded or ultraexpanded\" ",
		      -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "scale font element", VALUE_COLUMN,
		      "scale=\"xx_small, x_small, small, large, x_large, xx_large\" ",
		      -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "size font element", VALUE_COLUMN, "size=\"20\" ", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "margin left font element", VALUE_COLUMN,
		      "left=\"20\" ", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "margin right font element", VALUE_COLUMN,
		      "right=\"20\" ", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "foreground font element", VALUE_COLUMN,
		      "foreground=\"COLOR\" ", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "background font element", VALUE_COLUMN,
		      "background=\"COLOR\" ", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "justify font element", VALUE_COLUMN,
		      "justify=\"LEFT, RIGHT or CENTER\" ", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "underline font element", VALUE_COLUMN,
		      "underline=\"SINGLE or DOUBLE\" ", -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c5, &iter_c4);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c5, TITLE_COLUMN,
		      "face font element", VALUE_COLUMN, "face=\"FONT\" ",
		      -1);

  gtk_tree_store_append (GTK_TREE_STORE (tagslist_model), &iter_c4, &iter_c3);
  gtk_tree_store_set (GTK_TREE_STORE (tagslist_model), &iter_c4, TITLE_COLUMN,
		      "Link tag", VALUE_COLUMN, "<link id=ID>TEXT</link>", -1);


}

/* EOF */
