/* gInfo - Copyright (C) 2004 bakunin - Andrea Marchesini 
 *                                      <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "ginfo.h"

typedef enum
{
  ITEMIZED,
  VARIABLE,
  ORDERED
} docbook_list_enum;

static docbook_list_enum docbook_list_type = ITEMIZED;
static int docbook_list_ordered_n = 0;
static int docbook_no_trim = 0;
static char *docbook_link = NULL;

#define INDENT 15

void modules_docbook_free (module *);
int modules_docbook_check_title (xmlDocPtr, xmlNodePtr, module *);
int modules_docbook_check_secs (void *, xmlDocPtr, xmlNodePtr, module *);
void modules_docbook_check_secs_rec (xmlDocPtr, xmlNodePtr,
				     module_xml_info **, module_xml_info **,
				     module_xml_info_font *);

void modules_docbook_info_add (module_xml_info **, module_xml_info **,
			       module_xml_info *);
void modules_docbook_info_add_text (module_xml_info **, module_xml_info **,
				    char *);
void modules_docbook_info_add_font (module_xml_info **, module_xml_info **,
				    module_xml_info_font *);
void modules_docbook_info_add_nl (module_xml_info **, module_xml_info **);
void modules_docbook_info_add_title (module_xml_info **, module_xml_info **,
				     xmlDocPtr, xmlNodePtr, int,
				     module_xml_info_font *);
void modules_docbook_info_add_link (module_xml_info **, module_xml_info **,
				    module_xml_info_font *, char *);

int
modules_docbook_check (xmlDocPtr doc, xmlNodePtr c, char *a)
{

  module *m;
  xmlNodePtr cur;

  for (; c != NULL; c = c->next)
    {

      void *t = NULL;
      ginfo_docs *docs = NULL;
      int docs_n = 0;

      if (!(m = (module *) malloc (sizeof (module))))
	fatal ("Error: memory.");

      memset (m, 0, sizeof (module));

      m->type = GINFO_XML;
      m->genre = GINFO_GENRE_OTHER;
      m->handle = doc;

      if (!(m->file = strdup (a)))
	fatal ("Error: memory.");

      for (cur = c->children; cur != NULL; cur = cur->next)
	{

	  if (!xmlStrcmp (cur->name, "articleinfo"))
	    {
	      if (modules_docbook_check_title (doc, cur, m))
		break;
	    }

	  else if (!xmlStrcmp (cur->name, "sect1"))
	    {
	      if (!t)
		t = ginfo_easy_docs_init ();

	      if (modules_docbook_check_secs (t, doc, cur, m))
		{
		  ginfo_easy_docs_free (t);
		  break;
		}
	    }
	}

      if (t)
	docs = ginfo_easy_docs_close (t, &docs_n);

      if (!m->name)
	{
	  msg ("Module %s: no name.", m->file);
	  modules_docbook_free (m);
	  continue;
	}

      if (!docs)
	{
	  msg ("Module %s: no docs.", m->file);
	  modules_docbook_free (m);
	  continue;
	}

      if (docs_add (docs, docs_n, m))
	{
	  msg ("Module %s: no docs.", m->file);
	  modules_docbook_free (m);
	  ginfo_easy_docs_list_free (docs);
	  continue;
	}

      m->next = module_head;
      module_head = m;

    }

  return 0;
}

void
modules_docbook_free (module * m)
{
  doc *d;
  module_xml_info *i, *j;

  if (m->file)
    free (m->file);
  if (m->name)
    free (m->name);

  while (m->docs)
    {

      i = m->docs->arg;
      while (i)
	{
	  j = i->next;
	  free (i);
	  i = j;
	}

      d = m->docs->next;
      free (m->docs);
      m->docs = d;
    }

  free (m);
}

int
modules_docbook_check_title (xmlDocPtr doc, xmlNodePtr cur, module * m)
{

  for (cur = cur->children; cur != NULL; cur = cur->next)
    {
      if (!xmlStrcmp (cur->name, "title"))
	{
	  if (!(m->name = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1))
	      || !*m->name)
	    {
	      msg ("Module %s: no name.", m->file);
	      return 1;
	    }
	}
    }

  if (!m->name)
    return 1;

  return 0;
}

int
modules_docbook_check_secs (void *t, xmlDocPtr doc, xmlNodePtr c, module * m)
{
  char *chapter = NULL;
  char *id = NULL;

  xmlNodePtr cur;

  module_xml_info *tail = NULL;
  module_xml_info *head = NULL;
  module_xml_info_font font;

  memset (&font, 0x0, sizeof (font));
  font.scale = PANGO_SCALE_MEDIUM;

  id = xmlGetProp (c, "id");
  if (!id || !*id)
    id = NULL;

  for (cur = c->children; cur != NULL; cur = cur->next)
    {
      if (!xmlStrcmp (cur->name, "title"))
	{
	  if (chapter
	      || !(chapter =
		   xmlNodeListGetString (doc, cur->xmlChildrenNode, 1))
	      || !*chapter)
	    {
	      msg ("Module %s: syntax error title tag.", m->file);
	      return 1;
	    }

	  chapter = trim (chapter);
	  modules_docbook_info_add_title (&head, &tail, doc, c, 1, &font);
	}
      else
	{
	  modules_docbook_check_secs_rec (doc, cur, &head, &tail, &font);
	}
    }

  if (head)
    {
      ginfo_easy_docs_add (t, chapter, ginfo_tutor_xml, (void *) head, id);
      return 0;
    }

  return 1;
}

void
modules_docbook_check_secs_rec (xmlDocPtr doc, xmlNodePtr cur,
				module_xml_info ** h, module_xml_info ** t,
				module_xml_info_font * font)
{
  int no_rec = 0;
  int nl = 0, i;
  docbook_list_enum list_type = docbook_list_type;
  int list_ordered_n = docbook_list_ordered_n;
  int no_trim = docbook_no_trim;
  char *link = docbook_link;

  module_xml_info_font f;
  memcpy (&f, font, sizeof (f));

  if (cur->type == XML_TEXT_NODE)
    {
      char *text = xmlNodeGetContent (cur);
      if (!text)
	return;

      if (!no_trim)
	if (!(text = modules_xml_info_trim (text)))
	  return;

      modules_docbook_info_add_text (h, t, text);
      return;
    }

  if (cur->type == XML_ELEMENT_NODE)
    {

      /* SECTION TAGS */
      if (!strcmp (cur->name, "sect1"))
	modules_docbook_info_add_title (h, t, doc, cur, 1, font);

      else if (!strcmp (cur->name, "sect2"))
	modules_docbook_info_add_title (h, t, doc, cur, 2, font);

      else if (!strcmp (cur->name, "sect3"))
	modules_docbook_info_add_title (h, t, doc, cur, 3, font);

      else if (!strcmp (cur->name, "sect4"))
	modules_docbook_info_add_title (h, t, doc, cur, 4, font);

      else if (!strcmp (cur->name, "sect5"))
	modules_docbook_info_add_title (h, t, doc, cur, 5, font);

      else if (!strcmp (cur->name, "sect6"))
	modules_docbook_info_add_title (h, t, doc, cur, 6, font);

      /* PARA TAG */
      else if (!strcmp (cur->name, "para"))
	{
	  nl = 1;
	}

      /* COMMAND TAGS */
      else if (!strcmp (cur->name, "command")
	       || !strcmp (cur->name, "computeroutput")
	       || !strcmp (cur->name, "email")
	       || !strcmp (cur->name, "filename")
	       || !strcmp (cur->name, "literal")
	       || !strcmp (cur->name, "option")
	       || !strcmp (cur->name, "replaceable")
	       || !strcmp (cur->name, "systemitem")
	       || !strcmp (cur->name, "userinput"))
	{
	  f.stretch = PANGO_STRETCH_EXTRA_EXPANDED;
	  f.face = "monospace";

	  modules_docbook_info_add_font (h, t, &f);
	}

      /* BLOCKQUOTE TAG */
      else if (!strcmp (cur->name, "blockquote"))
	{
	  f.margin_left = 50;
	  f.margin_right = 50;

	  nl = 1;

	  modules_docbook_info_add_font (h, t, &f);
	}

      /* NOTE TAG */
      else if (!strcmp (cur->name, "note"))
	{
	  f.margin_left = 50;
	  f.margin_right = 50;
	  f.style = PANGO_STYLE_OBLIQUE;

	  nl = 1;

	  modules_docbook_info_add_font (h, t, &f);
	}

      /* ACRONYM TAG */
      else if (!strcmp (cur->name, "acronym")
	       || !strcmp (cur->name, "emphasis")
	       || !strcmp (cur->name, "trademark"))
	{
	  f.weight = PANGO_WEIGHT_BOLD;

	  modules_docbook_info_add_font (h, t, &f);
	}

      /* SCREEN TAG */
      else if (!strcmp (cur->name, "screen"))
	{
	  docbook_no_trim = 1;
	}

      /* LISTITEM TAGS */
      else if (!strcmp (cur->name, "listitem")
	       && (docbook_list_type == ITEMIZED
		   || docbook_list_type == ORDERED))
	{
	  modules_docbook_info_add_nl (h, t);

	  f.weight = PANGO_WEIGHT_BOLD;
	  f.margin_left += INDENT;

	  modules_docbook_info_add_font (h, t, &f);

	  if (docbook_list_type == ORDERED)
	    {
	      char s[20];
	      char *s2;

	      snprintf (s, 20, " %d - ", docbook_list_ordered_n);
	      docbook_list_ordered_n++;

	      if (!(s2 = strdup (s)))
		fatal ("Error: memory.");

	      modules_docbook_info_add_text (h, t, s2);

	    }
	  else
	    modules_docbook_info_add_text (h, t, " * ");

	  memcpy (&f, font, sizeof (f));
	  f.margin_left += INDENT;
	  modules_docbook_info_add_font (h, t, &f);
	}

      else if (!strcmp (cur->name, "orderedlist"))
	{
	  docbook_list_type = ORDERED;
	  docbook_list_ordered_n = 1;
	}

      else if (!strcmp (cur->name, "varlistentry"))
	{
	  xmlNodePtr child;

	  modules_docbook_info_add_nl (h, t);

	  f.weight = PANGO_WEIGHT_BOLD;
	  f.margin_left += INDENT;

	  modules_docbook_info_add_font (h, t, &f);

	  for (child = cur->children; child != NULL; child = child->next)
	    {
	      if (!strcmp (child->name, "term"))
		{
		  char *text = xmlNodeGetContent (child);

		  if (!text)
		    return;

		  if (!(text = modules_xml_info_trim (text)))
		    return;

		  modules_docbook_info_add_text (h, t, text);
		}
	    }

	  memcpy (&f, font, sizeof (f));
	  f.margin_left += INDENT;
	  modules_docbook_info_add_font (h, t, &f);
	  modules_docbook_info_add_text (h, t, " - ");

	  docbook_list_type = VARIABLE;
	}

      /* LINK TAG */
      else if (!strcmp (cur->name, "link"))
	{
	  docbook_link = xmlGetProp (cur, "linkend");

	  if (docbook_link)
	    modules_docbook_info_add_link (h, t, &f, docbook_link);
	}

      /* IGNORED TAGS and NO_RECURSIVITY */
      else if (!strcmp (cur->name, "title")	/* used by add_title */
	       || !strcmp (cur->name, "term")	/* used by variablelist */
	)
	{
	  no_rec = 1;
	}

      /* IGNORED TAGS */
      else if (!strcmp (cur->name, "itemizedlist")	/* I use listitem */
	       || !strcmp (cur->name, "variablelist")	/* I use varlistentry */
	       || !strcmp (cur->name, "listitem")	/* checked for other 
							   type of list if
							   docbook_list_type
							   != ITEMIZED */
	)
	{
	}

      else
	msg ("Ignored tag: %s", cur->name);

      if (!no_rec)
	for (cur = cur->children; cur != NULL; cur = cur->next)
	  modules_docbook_check_secs_rec (doc, cur, h, t, &f);

      for (i = 0; i < nl; i++)
	modules_docbook_info_add_nl (h, t);

      if (memcmp (&f, font, sizeof (f)))
	modules_docbook_info_add_font (h, t, font);

      docbook_list_type = list_type;

      if (docbook_list_type != ORDERED)
	docbook_list_ordered_n = list_ordered_n;

      docbook_no_trim = no_trim;
      docbook_link = link;
    }
}

void
modules_docbook_info_add (module_xml_info ** h, module_xml_info ** t,
			  module_xml_info * add)
{
  if (!*h)
    *h = add;
  else
    (*t)->next = add;

  *t = add;
}

void
modules_docbook_info_add_text (module_xml_info ** h, module_xml_info ** t,
			       char *text)
{
  module_xml_info *c;

  if (!(c = (module_xml_info *) malloc (sizeof (module_xml_info))))
    fatal ("Error: memory.");

  c->next = NULL;
  c->type = INFO_TEXT;
  c->text = text;
  c->link = docbook_link;

  modules_docbook_info_add (h, t, c);
}

void
modules_docbook_info_add_font (module_xml_info ** h, module_xml_info ** t,
			       module_xml_info_font * font)
{
  module_xml_info *c;

  if (!(c = (module_xml_info *) malloc (sizeof (module_xml_info))))
    fatal ("Error: memory.");

  c->next = NULL;
  c->type = INFO_FONT;
  c->text = NULL;
  c->link = docbook_link;

  memcpy (&c->font, font, sizeof (module_xml_info_font));

  modules_docbook_info_add (h, t, c);
}

void
modules_docbook_info_add_nl (module_xml_info ** h, module_xml_info ** t)
{

  module_xml_info *c;

  if (!(c = (module_xml_info *) malloc (sizeof (module_xml_info))))
    fatal ("Error: memory.");

  c->next = NULL;
  c->text = NULL;
  c->link = docbook_link;

  c->type = INFO_NEWLINE;

  modules_docbook_info_add (h, t, c);
}

void
modules_docbook_info_add_title (module_xml_info ** h, module_xml_info ** t,
				xmlDocPtr doc, xmlNodePtr cur, int value,
				module_xml_info_font * font)
{
  char *title;
  module_xml_info_font f;

  modules_docbook_info_add_nl (h, t);
  modules_docbook_info_add_nl (h, t);

  memset (&f, 0, sizeof (f));

  switch (value)
    {
    case 1:
      f.scale = PANGO_SCALE_XX_LARGE;
      f.justify = GTK_JUSTIFY_CENTER;
      f.weight = PANGO_WEIGHT_ULTRABOLD;
      break;

    case 2:
      f.scale = PANGO_SCALE_X_LARGE;
      f.justify = GTK_JUSTIFY_CENTER;
      f.weight = PANGO_WEIGHT_ULTRABOLD;
      break;

    case 3:
      f.scale = PANGO_SCALE_LARGE;
      f.justify = GTK_JUSTIFY_CENTER;
      f.weight = PANGO_WEIGHT_BOLD;
      break;

    case 4:
      f.scale = PANGO_SCALE_MEDIUM;
      f.justify = GTK_JUSTIFY_CENTER;
      f.weight = PANGO_WEIGHT_BOLD;
      f.style = PANGO_STYLE_ITALIC;
      break;

    case 5:
      f.scale = PANGO_SCALE_MEDIUM;
      f.justify = GTK_JUSTIFY_CENTER;
      f.weight = PANGO_WEIGHT_BOLD;
      break;

    case 6:
      f.scale = PANGO_SCALE_MEDIUM;
      f.justify = GTK_JUSTIFY_CENTER;
      break;

    default:
      f.scale = PANGO_SCALE_MEDIUM;
      break;
    }

  modules_docbook_info_add_font (h, t, &f);

  for (cur = cur->children; cur != NULL; cur = cur->next)
    {
      if (!strcmp (cur->name, "title"))
	{
	  if (!(title = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1))
	      || !*title)
	    break;

	  modules_docbook_info_add_text (h, t, title);
	  break;
	}
    }

  modules_docbook_info_add_font (h, t, font);

  modules_docbook_info_add_nl (h, t);
  modules_docbook_info_add_nl (h, t);
}

void
modules_docbook_info_add_link (module_xml_info ** h, module_xml_info ** t,
			       module_xml_info_font * font, char *link)
{

  module_xml_info *c;

  if (!(c = (module_xml_info *) malloc (sizeof (module_xml_info))))
    fatal ("Error: memory.");

  c->next = NULL;
  c->text = NULL;

  c->type = INFO_FONT;

  c->link = link;

  memcpy (&c->font, font, sizeof (module_xml_info_font));

  modules_docbook_info_add (h, t, c);
}

/* EOF */
