/* gInfo - Copyright (C) 2004 bakunin - Andrea Marchesini 
 *                                      <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "ginfo.h"

int
modules_c_check (char *a)
{
  void *lib;
  int (*func0) (void);
  char *(*func1) (void);
  int (*func2) (void);
  char **(*func3) (void);
  ginfo_tips *(*func4) (int *);
  ginfo_docs *(*func5) (int *);
  int d = 0;
  char *name = NULL;
  int genre;
  char **words = NULL;
  ginfo_tips *tooltips = NULL;
  ginfo_docs *docs = NULL;
  int n;
  module *m;

  if (!(lib = dlopen (a, RTLD_LAZY)))
    {
      msg (dlerror ());
      return 1;
    }

  if ((func0 = dlsym (lib, "module_default")))
    {
      d = func0 ();
    }

  if (!(func1 = dlsym (lib, "module_name")))
    {
      dlclose (lib);
      return 1;
    }

  if (!(name = func1 ()))
    {
      dlclose (lib);
      return 1;
    }

  if (!(func2 = dlsym (lib, "module_genre")))
    {
      dlclose (lib);
      return 1;
    }

  if (!(func3 = dlsym (lib, "module_words")))
    {
      dlclose (lib);
      return 1;
    }

  if (!(words = func3 ()))
    {
      dlclose (lib);
      return 1;
    }

  if (!(func5 = dlsym (lib, "module_docs")) || !(docs = func5 (&n)))
    {
      dlclose (lib);
      return 1;
    }

  if (!(m = (module *) malloc (sizeof (module))))
    fatal ("Error: memory.");

  m->docs = NULL;
  m->type = GINFO_C;

  if (docs_add (docs, n, m))
    {
      dlclose (lib);
      free (m);
      return 1;
    }

  if (!(m->file = strdup (a)))
    fatal ("Error: memory");

  m->handle = lib;
  m->name = name;
  m->genre = genre;
  m->words = words;

  m->next = module_head;
  module_head = m;

  if (d)
    {
      modules_set_default (a, m);
    }

  if ((func4 = dlsym (lib, "module_tooltips")) != NULL
      && (tooltips = func4 (&n)) != NULL)
    tooltips_add (tooltips, n, m);

  return 0;
}

/* EOF */
