/* gInfo - Copyright (C) 2004 bakunin - Andrea Marchesini 
 *                                      <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "ginfo.h"
#include "ginfo_logo_s.xpm"

#define getrandom(min, max) ((rand() % (int)(((max)+1) - (min))) + (min))

typedef struct
{
  GtkWidget *(*func) (void *);
  void *arg;
  char *id;

  module *module;

} tooltip_struct;

tooltip *tooltip_head = NULL;
int tooltip_time = 5;
int tooltip_done = 0;
int tooltip_change_time = 0;
int tooltip_request = 0;
int tooltip_number = 0;

static GtkWidget *tooltip_window = NULL;
static GtkWidget *tooltip_checkbutton;

void tooltips_dialog (char *, module *, void *, void *, char *id);
void tooltips_no (GtkWidget *, tooltip_struct *);
void tooltips_yes (GtkWidget * w, tooltip_struct *);
int tooltips_check_single_id (char *);
doc *tooltips_find_id (char *);

void
tooltips_add (ginfo_tips * list, int n, module * m)
{
  tooltip *t;

  while (n)
    {
      if (!(t = (tooltip *) malloc (sizeof (tooltip))))
	fatal ("Error: memory.");

      t->module = m;
      t->tooltip = list->text;
      t->func = list->func;
      t->arg = list->arg;
      t->id = list->id;

      list++;
      n--;

      t->next = tooltip_head;
      tooltip_head = t;

      tooltip_number++;
    }
}

void
tooltips_timer (void)
{

  srand (getpid ());

  g_timeout_add (tooltip_time * 60000, tooltips_timeout, NULL);
}

int
tooltips_timeout (gpointer dummy)
{

  int k = getrandom (1, tooltip_number);
  tooltip *t = tooltip_head;

  if (tooltip_change_time)
    {
      g_timeout_add (tooltip_time * 60000, tooltips_timeout, NULL);
      tooltip_change_time = 0;
      return 0;
    }

  if (tooltip_done && !tooltip_request)
    return 1;

  if (tooltip_request)
    tooltip_request = 0;

  while (t)
    {
      k--;

      if (!k)
	{
	  tooltips_dialog (t->tooltip, t->module, (void *) t->func, t->arg,
			   t->id);
	  break;
	}

      t = t->next;
    }

  return 1;
}

void
tooltips_dialog (char *text, module * m, void *f, void *arg, char *id)
{
  GtkWidget *box;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GdkPixmap *pixmap;
  GdkPixmap *mask;
  GtkWidget *image = NULL;
  GtkWidget *label;
  GtkWidget *button;
  char buf[255];
  tooltip_struct *ts;

  if (!(ts = (tooltip_struct *) malloc (sizeof (tooltip_struct))))
    fatal ("Error: memory.");

  ts->func = f;
  ts->arg = arg;
  ts->module = m;
  ts->id = id;

  if (tooltip_window)
    return;

  snprintf (buf, 255, "%s %s - tips!", PACKAGE, VERSION);

  tooltip_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (tooltip_window), buf);
  gtk_window_set_position (GTK_WINDOW (tooltip_window), GTK_WIN_POS_CENTER);
  gtk_widget_realize (tooltip_window);

  g_signal_connect ((gpointer) tooltip_window, "destroy",
		    G_CALLBACK (tooltips_no), NULL);

  vbox = gtk_vbox_new (FALSE, 5);
  gtk_widget_show (vbox);
  gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
  gtk_container_add (GTK_CONTAINER (tooltip_window), vbox);

  hbox = gtk_hbox_new (FALSE, 5);
  gtk_widget_show (hbox);
  gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);

  box = gtk_vbox_new (FALSE, 5);
  gtk_widget_show (box);
  gtk_container_set_border_width (GTK_CONTAINER (box), 5);
  gtk_box_pack_start (GTK_BOX (hbox), box, TRUE, TRUE, 0);

  label = gtk_label_new (text);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);

  snprintf (buf, 255, "More info about %s?", m->name);
  label = gtk_label_new (buf);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);

  if (graphic_image)
    {
      if (!(image = gtk_image_new_from_file (graphic_image)))
	msg ("Could not find image %s.", graphic_image);
    }

  if (!image)
    {
      pixmap =
	gdk_pixmap_create_from_xpm_d (tooltip_window->window, &mask, NULL,
				      ginfo_logo_s);
      image = gtk_image_new_from_pixmap (pixmap, mask);
    }

  gtk_widget_show (image);
  gtk_box_pack_start (GTK_BOX (hbox), image, TRUE, TRUE, 0);

  hbox = gtk_hbox_new (FALSE, 5);
  gtk_widget_show (hbox);
  gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);

  tooltip_checkbutton =
    gtk_check_button_new_with_mnemonic ("Show tip next time");
  gtk_widget_show (tooltip_checkbutton);
  if (!tooltip_done)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tooltip_checkbutton),
				  TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tooltip_checkbutton),
				  FALSE);

  gtk_box_pack_start (GTK_BOX (hbox), tooltip_checkbutton, TRUE, TRUE, 0);

  button = gtk_button_new_from_stock ("gtk-yes");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);

  g_signal_connect ((gpointer) button, "clicked", G_CALLBACK (tooltips_yes),
		    ts);

  button = gtk_button_new_from_stock ("gtk-no");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);

  g_signal_connect ((gpointer) button, "clicked", G_CALLBACK (tooltips_no),
		    ts);

  gtk_widget_show (tooltip_window);
}

void
tooltips_no (GtkWidget * w, tooltip_struct * ts)
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tooltip_checkbutton))
      == TRUE)
    tooltip_done = 0;
  else
    tooltip_done = 1;

  gtk_widget_hide (tooltip_window);
  gtk_widget_destroy (tooltip_window);

  tooltip_window = NULL;

  free (ts);

  config_write ();
}

void
tooltips_yes (GtkWidget * w, tooltip_struct * ts)
{
  if (ts->id)
    {
      doc *d = tooltips_find_id (ts->id);
      tutor_container (d->func, d->arg);
    }
  else if (ts->func)
    {
      tutor_container (ts->func, ts->arg);
    }
  else if (ts->module && ts->module->docs && ts->module->docs->func)
    {
      tutor_container (ts->module->docs->func, ts->module->docs->arg);
    }

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tooltip_checkbutton))
      == TRUE)
    tooltip_done = 0;
  else
    tooltip_done = 1;

  gtk_widget_hide (tooltip_window);
  gtk_widget_destroy (tooltip_window);

  tooltip_window = NULL;

  free (ts);

  config_write ();
}

void
tooltips_close (void)
{
  tooltip *t;

  t = tooltip_head;

  while (t)
    {
      tooltip_head = t->next;

      free (t);
      t = tooltip_head;
    }
}

void
tooltips_check_id (void)
{
  tooltip *t;

  t = tooltip_head;

  while (t)
    {
      if (t->id)
	if (tooltips_check_single_id (t->id))
	  t->id = NULL;

      t = t->next;
    }
}

int
tooltips_check_single_id (char *id)
{
  module *m = module_head;
  doc *d;

  while (m)
    {
      d = m->docs;

      while (d)
	{
	  if (d->id)
	    if (!strcmp (id, d->id))
	      return 0;

	  d = d->next;
	}
      m = m->next;
    }

  return 1;
}

doc *
tooltips_find_id (char *id)
{
  module *m = module_head;
  doc *d;

  while (m)
    {
      d = m->docs;

      while (d)
	{
	  if (d->id)
	    if (!strcmp (id, d->id))
	      return d;

	  d = d->next;
	}
      m = m->next;
    }

  return NULL;
}

/* EOF */
